@@ -14,6 +14,7 @@ from .._exceptions import APIConnectionError
 from ..log import logger
 from ..types import DEFAULT_API_CONNECT_OPTIONS, USERDATA_TIMED_TRANSCRIPT, APIConnectOptions
 from ..utils import aio
+from .stream_adapter import StreamAdapter
 from .tts import (
     TTS,
     AudioEmitter,
@@ -82,7 +83,7 @@ class FallbackAdapter(
 
         super().__init__(
             capabilities=TTSCapabilities(
-                streaming=all(t.capabilities.streaming for t in tts),
+                streaming=any(t.capabilities.streaming for t in tts),
                 aligned_transcript=all(t.capabilities.aligned_transcript for t in tts),
             ),
             sample_rate=sample_rate,
@@ -105,6 +106,14 @@ class FallbackAdapter(
 
             t.on("metrics_collected", self._on_metrics_collected)
 
+    @property
+    def model(self) -> str:
+        return "FallbackAdapter"
+
+    @property
+    def provider(self) -> str:
+        return "livekit"
+
     def synthesize(
         self, text: str, *, conn_options: APIConnectOptions = DEFAULT_FALLBACK_API_CONNECT_OPTIONS
     ) -> FallbackChunkedStream:
@@ -266,7 +275,17 @@ class FallbackSynthesizeStream(SynthesizeStream):
         conn_options: APIConnectOptions,
         recovering: bool = False,
     ) -> AsyncGenerator[SynthesizedAudio, None]:
-        stream = tts.stream(conn_options=conn_options)
+        # If TTS doesn't support streaming, wrap it with StreamAdapter
+        if tts.capabilities.streaming:
+            stream = tts.stream(conn_options=conn_options)
+        else:
+            from .. import tokenize
+
+            wrapped_tts = StreamAdapter(
+                tts=tts,
+                sentence_tokenizer=tokenize.blingfire.SentenceTokenizer(retain_format=True),
+            )
+            stream = wrapped_tts.stream(conn_options=conn_options)
 
         @utils.log_exceptions(logger=logger)
         async def _forward_input_task() -> None:
