@@ -14,6 +14,7 @@ from opentelemetry import trace
 from pydantic import BaseModel, ConfigDict, Field
 
 from livekit import rtc
+from livekit.agents.metrics.base import Metadata
 
 from .._exceptions import APIError
 from ..log import logger
@@ -84,6 +85,30 @@ class TTS(
     def label(self) -> str:
         return self._label
 
+    @property
+    def model(self) -> str:
+        """Get the model name/identifier for this TTS instance.
+
+        Returns:
+            The model name if available, "unknown" otherwise.
+
+        Note:
+            Plugins should override this property to provide their model information.
+        """
+        return "unknown"
+
+    @property
+    def provider(self) -> str:
+        """Get the provider name/identifier for this TTS instance.
+
+        Returns:
+            The provider name if available, "unknown" otherwise.
+
+        Note:
+            Plugins should override this property to provide their provider information.
+        """
+        return "unknown"
+
     @property
     def capabilities(self) -> TTSCapabilities:
         return self._capabilities
@@ -194,6 +219,7 @@ class ChunkedStream(ABC):
             cancelled=self._synthesize_task.cancelled(),
             label=self._tts._label,
             streamed=False,
+            metadata=Metadata(model_name=self._tts.model, model_provider=self._tts.provider),
         )
         if self._tts_request_span:
             self._tts_request_span.set_attribute(
@@ -237,8 +263,8 @@ class ChunkedStream(ABC):
                 # wait for all audio frames to be pushed & propagate errors
                 await output_emitter.join()
 
-                if output_emitter.pushed_duration() <= 0.0:
-                    raise APIError("no audio frames were pushed")
+                if self._input_text.strip() and output_emitter.pushed_duration() <= 0.0:
+                    raise APIError(f"no audio frames were pushed for text: {self._input_text}")
 
                 current_span.set_attribute(trace_types.ATTR_TTS_INPUT_TEXT, self._input_text)
                 return
@@ -444,6 +470,7 @@ class SynthesizeStream(ABC):
                 cancelled=self._task.cancelled(),
                 label=self._tts._label,
                 streamed=True,
+                metadata=Metadata(model_name=self._tts.model, model_provider=self._tts.provider),
             )
             if self._tts_request_span:
                 self._tts_request_span.set_attribute(
@@ -738,6 +765,31 @@ class AudioEmitter:
         debug_frames: list[rtc.AudioFrame] = []
         timed_transcripts: list[TimedString] = []
 
+        flush_timer: asyncio.TimerHandle | None = None
+        sent_start: float | None = None
+        sent_duration: float = 0.0
+        event_loop = asyncio.get_event_loop()
+
+        def _send_audio(ev: SynthesizedAudio, *, flush_if_delayed: bool = False) -> None:
+            nonlocal sent_start, sent_duration, flush_timer
+
+            self._dst_ch.send_nowait(ev)
+            if sent_start is None:
+                sent_start = event_loop.time()
+            sent_duration += ev.frame.duration
+
+            if flush_timer is not None:
+                flush_timer.cancel()
+
+            def _flush() -> None:
+                self.flush()
+                logger.debug("flush audio emitter due to slow audio generation")
+
+            if flush_if_delayed:
+                # force flush the buffer if the audio comes slower than realtime
+                delay = sent_duration - (event_loop.time() - sent_start) - 0.02
+                flush_timer = event_loop.call_later(delay, _flush)
+
         def _emit_frame(frame: rtc.AudioFrame | None = None, *, is_final: bool = False) -> None:
             nonlocal last_frame, segment_ctx, timed_transcripts
             assert segment_ctx is not None
@@ -764,26 +816,28 @@ class AudioEmitter:
                             debug_frames.append(frame)
 
                     frame.userdata[USERDATA_TIMED_TRANSCRIPT] = timed_transcripts
-                    self._dst_ch.send_nowait(
+                    _send_audio(
                         SynthesizedAudio(
                             frame=frame,
                             request_id=self._request_id,
                             segment_id=segment_ctx.segment_id,
                             is_final=True,
-                        )
+                        ),
+                        flush_if_delayed=False,
                     )
                     timed_transcripts = []
                     return
 
             if last_frame is not None:
                 last_frame.userdata[USERDATA_TIMED_TRANSCRIPT] = timed_transcripts
-                self._dst_ch.send_nowait(
+                _send_audio(
                     SynthesizedAudio(
                         frame=last_frame,
                         request_id=self._request_id,
                         segment_id=segment_ctx.segment_id,
                         is_final=is_final,
-                    )
+                    ),
+                    flush_if_delayed=not is_final,
                 )
                 timed_transcripts = []
                 segment_ctx.audio_duration += last_frame.duration
@@ -796,19 +850,21 @@ class AudioEmitter:
 
         def _flush_frame() -> None:
             nonlocal last_frame, segment_ctx, timed_transcripts
+            nonlocal flush_timer, sent_start, sent_duration
             assert segment_ctx is not None
 
             if last_frame is None:
                 return
 
             last_frame.userdata[USERDATA_TIMED_TRANSCRIPT] = timed_transcripts
-            self._dst_ch.send_nowait(
+            _send_audio(
                 SynthesizedAudio(
                     frame=last_frame,
                     request_id=self._request_id,
                     segment_id=segment_ctx.segment_id,
                     is_final=False,  # flush isn't final
-                )
+                ),
+                flush_if_delayed=False,  # don't flush again before new frames are pushed
             )
             timed_transcripts = []
             segment_ctx.audio_duration += last_frame.duration
@@ -818,6 +874,12 @@ class AudioEmitter:
                 debug_frames.append(last_frame)
 
             last_frame = None
+            # reset sent duration after flush
+            sent_start = None
+            sent_duration = 0.0
+            if flush_timer is not None:
+                flush_timer.cancel()
+                flush_timer = None
 
         def dump_segment() -> None:
             nonlocal segment_ctx
@@ -925,11 +987,12 @@ class AudioEmitter:
                             decode_atask = asyncio.create_task(_decode_task())
                         audio_decoder.push(data)
                     elif decode_atask:
-                        if isinstance(data, AudioEmitter._FlushSegment) and audio_decoder:
-                            audio_decoder.end_input()
-                            await decode_atask
-                            _flush_frame()
-                            audio_decoder = None
+                        if isinstance(data, AudioEmitter._FlushSegment):
+                            if audio_decoder:
+                                audio_decoder.end_input()
+                                await decode_atask
+                                _flush_frame()
+                                audio_decoder = None
 
                         elif isinstance(data, AudioEmitter._EndSegment) and segment_ctx:
                             if audio_decoder:
@@ -942,6 +1005,9 @@ class AudioEmitter:
                             logger.warning("unknown data type: %s", type(data))
 
         finally:
+            if flush_timer is not None:
+                flush_timer.cancel()
+
             if audio_decoder and decode_atask:
                 await audio_decoder.aclose()
                 await aio.cancel_and_wait(decode_atask)
