@@ -5,6 +5,8 @@ from typing import Optional
 
 import psutil
 
+from ...log import logger
+
 
 class CPUMonitor(ABC):
     @abstractmethod
@@ -20,9 +22,18 @@ class CPUMonitor(ABC):
         pass
 
 
+def _cpu_count_from_env() -> Optional[float]:
+    try:
+        if "NUM_CPUS" in os.environ:
+            return float(os.environ["NUM_CPUS"])
+    except ValueError:
+        logger.warning("Failed to parse NUM_CPUS from environment", exc_info=True)
+    return None
+
+
 class DefaultCPUMonitor(CPUMonitor):
     def cpu_count(self) -> float:
-        return psutil.cpu_count() or 1.0
+        return _cpu_count_from_env() or psutil.cpu_count() or 1.0
 
     def cpu_percent(self, interval: float = 0.5) -> float:
         return psutil.cpu_percent(interval) / 100.0
@@ -34,9 +45,12 @@ class CGroupV2CPUMonitor(CPUMonitor):
         # period: The period of time in microseconds over which the quota applies.
         # If the quota is set to "max", it means the cgroup is allowed to use all available CPUs without restriction.  # noqa: E501
         # Otherwise, the quota is a number that represents the maximum CPU time in microseconds that the cgroup can use within a given period.  # noqa: E501
+        env_cpus = _cpu_count_from_env()
+        if env_cpus is not None:
+            return env_cpus
         quota, period = self._read_cpu_max()
         if quota == "max":
-            return os.cpu_count() or 1
+            return psutil.cpu_count() or 1.0
         return 1.0 * int(quota) / period
 
     def cpu_percent(self, interval: float = 0.5) -> float:
@@ -45,13 +59,10 @@ class CGroupV2CPUMonitor(CPUMonitor):
         cpu_usage_end = self._read_cpu_usage()
         cpu_usage_diff = cpu_usage_end - cpu_usage_start
 
-        # Convert microseconds to seconds
+        # microseconds to seconds
         cpu_usage_seconds = cpu_usage_diff / 1_000_000
 
-        # Get the number of CPUs available to the container
         num_cpus = self.cpu_count()
-
-        # Calculate the percentage
         cpu_usage_percent = cpu_usage_seconds / (interval * num_cpus)
 
         return min(cpu_usage_percent, 1)
@@ -77,9 +88,16 @@ class CGroupV2CPUMonitor(CPUMonitor):
 
 class CGroupV1CPUMonitor(CPUMonitor):
     def cpu_count(self) -> float:
+        # often, cgroups v1 quota isn't set correctly, so we need to rely on an env var to
+        # correctly determine the number of CPUs
+        env_cpus = _cpu_count_from_env()
+        if env_cpus is not None:
+            return env_cpus
         quota, period = self._read_cfs_quota_and_period()
         if quota is None or quota < 0 or period is None or period <= 0:
-            return os.cpu_count() or 1.0
+            # we do not want to use the node CPU count, as it could overstate the number
+            # available to the container
+            return 2.0
         return max(1.0 * quota / period, 1.0)
 
     def cpu_percent(self, interval: float = 0.5) -> float:
@@ -91,7 +109,7 @@ class CGroupV1CPUMonitor(CPUMonitor):
         usage_seconds = usage_diff_ns / 1_000_000_000
         num_cpus = self.cpu_count()
         percent = usage_seconds / (interval * num_cpus)
-        return min(percent, 1.0)
+        return max(min(percent, 1.0), 0.0)
 
     def _read_cfs_quota_and_period(self) -> tuple[Optional[int], Optional[int]]:
         quota_path_candidates = [
