@@ -4,7 +4,8 @@ import asyncio
 import copy
 import json
 import time
-from collections.abc import AsyncIterable
+from collections.abc import AsyncIterable, Sequence
+from contextlib import AbstractContextManager, nullcontext
 from dataclasses import asdict, dataclass
 from types import TracebackType
 from typing import (
@@ -14,6 +15,7 @@ from typing import (
     Protocol,
     TypeVar,
     Union,
+    overload,
     runtime_checkable,
 )
 
@@ -21,7 +23,7 @@ from opentelemetry import context as otel_context, trace
 
 from livekit import rtc
 
-from .. import llm, stt, tts, utils, vad
+from .. import inference, llm, stt, tts, utils, vad
 from ..cli import cli
 from ..job import get_job_context
 from ..llm import ChatContext
@@ -39,7 +41,6 @@ from .agent import Agent
 from .agent_activity import AgentActivity
 from .audio_recognition import _TurnDetector
 from .events import (
-    AgentFalseInterruptionEvent,
     AgentState,
     AgentStateChangedEvent,
     CloseEvent,
@@ -54,7 +55,9 @@ from .run_result import RunResult
 from .speech_handle import SpeechHandle
 
 if TYPE_CHECKING:
+    from ..inference import LLMModels, STTModels, TTSModels
     from ..llm import mcp
+    from .transcription.filters import TextTransforms
 
 
 @dataclass
@@ -76,10 +79,12 @@ class VoiceOptions:
     max_endpointing_delay: float
     max_tool_steps: int
     user_away_timeout: float | None
-    agent_false_interruption_timeout: float | None
+    false_interruption_timeout: float | None
+    resume_false_interruption: bool
     min_consecutive_speech_delay: float
-    use_tts_aligned_transcript: bool
+    use_tts_aligned_transcript: NotGivenOr[bool]
     preemptive_generation: bool
+    tts_text_transforms: Sequence[TextTransforms] | None
 
 
 Userdata_T = TypeVar("Userdata_T")
@@ -112,9 +117,6 @@ class _VideoSampler(Protocol):
 # TODO(theomonnom): Should this be moved to another file?
 class VoiceActivityVideoSampler:
     def __init__(self, *, speaking_fps: float = 1.0, silent_fps: float = 0.3):
-        if speaking_fps <= 0 or silent_fps <= 0:
-            raise ValueError("FPS values must be greater than zero")
-
         self.speaking_fps = speaking_fps
         self.silent_fps = silent_fps
         self._last_sampled_time: float | None = None
@@ -123,6 +125,8 @@ class VoiceActivityVideoSampler:
         now = time.time()
         is_speaking = session.user_state == "speaking"
         target_fps = self.speaking_fps if is_speaking else self.silent_fps
+        if target_fps == 0:
+            return False
         min_frame_interval = 1.0 / target_fps
 
         if self._last_sampled_time is None:
@@ -136,32 +140,39 @@ class VoiceActivityVideoSampler:
         return False
 
 
+DEFAULT_TTS_TEXT_TRANSFORMS: list[TextTransforms] = ["filter_markdown", "filter_emoji"]
+
+
 class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
     def __init__(
         self,
         *,
         turn_detection: NotGivenOr[TurnDetectionMode] = NOT_GIVEN,
-        stt: NotGivenOr[stt.STT] = NOT_GIVEN,
+        stt: NotGivenOr[stt.STT | STTModels | str] = NOT_GIVEN,
         vad: NotGivenOr[vad.VAD] = NOT_GIVEN,
-        llm: NotGivenOr[llm.LLM | llm.RealtimeModel] = NOT_GIVEN,
-        tts: NotGivenOr[tts.TTS] = NOT_GIVEN,
+        llm: NotGivenOr[llm.LLM | llm.RealtimeModel | LLMModels | str] = NOT_GIVEN,
+        tts: NotGivenOr[tts.TTS | TTSModels | str] = NOT_GIVEN,
         mcp_servers: NotGivenOr[list[mcp.MCPServer]] = NOT_GIVEN,
         userdata: NotGivenOr[Userdata_T] = NOT_GIVEN,
         allow_interruptions: bool = True,
         discard_audio_if_uninterruptible: bool = True,
         min_interruption_duration: float = 0.5,
         min_interruption_words: int = 0,
-        min_endpointing_delay: float = 0.4,
-        max_endpointing_delay: float = 6.0,
+        min_endpointing_delay: float = 0.5,
+        max_endpointing_delay: float = 3.0,
         max_tool_steps: int = 3,
         video_sampler: NotGivenOr[_VideoSampler | None] = NOT_GIVEN,
         user_away_timeout: float | None = 15.0,
-        agent_false_interruption_timeout: float | None = 4.0,
+        false_interruption_timeout: float | None = 2.0,
+        resume_false_interruption: bool = True,
         min_consecutive_speech_delay: float = 0.0,
-        use_tts_aligned_transcript: bool = False,
+        use_tts_aligned_transcript: NotGivenOr[bool] = NOT_GIVEN,
+        tts_text_transforms: NotGivenOr[Sequence[TextTransforms] | None] = NOT_GIVEN,
         preemptive_generation: bool = False,
         conn_options: NotGivenOr[SessionConnectOptions] = NOT_GIVEN,
         loop: asyncio.AbstractEventLoop | None = None,
+        # deprecated
+        agent_false_interruption_timeout: NotGivenOr[float | None] = NOT_GIVEN,
     ) -> None:
         """`AgentSession` is the LiveKit Agents runtime that glues together
         media streams, speech/LLM components, and tool orchestration into a
@@ -187,10 +198,10 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
                 If *NOT_GIVEN*, the session chooses the best available mode in
                 priority order ``realtime_llm → vad → stt → manual``; it
                 automatically falls back if the necessary model is missing.
-            stt (stt.STT, optional): Speech-to-text backend.
+            stt (stt.STT | str, optional): Speech-to-text backend.
             vad (vad.VAD, optional): Voice-activity detector
-            llm (llm.LLM | llm.RealtimeModel, optional): LLM or RealtimeModel
-            tts (tts.TTS, optional): Text-to-speech engine.
+            llm (llm.LLM | llm.RealtimeModel | str, optional): LLM or RealtimeModel
+            tts (tts.TTS | str, optional): Text-to-speech engine.
             mcp_servers (list[mcp.MCPServer], optional): List of MCP servers
                 providing external tools for the agent to use.
             userdata (Userdata_T, optional): Arbitrary per-session user data.
@@ -206,9 +217,9 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             min_endpointing_delay (float): Minimum time-in-seconds the agent
                 must wait after a potential end-of-utterance signal (from VAD
                 or an EOU model) before it declares the user’s turn complete.
-                Default ``0.4`` s.
+                Default ``0.5`` s.
             max_endpointing_delay (float): Maximum time-in-seconds the agent
-                will wait before terminating the turn. Default ``6.0`` s.
+                will wait before terminating the turn. Default ``3.0`` s.
             max_tool_steps (int): Maximum consecutive tool calls per LLM turn.
                 Default ``3``.
             video_sampler (_VideoSampler, optional): Uses
@@ -218,18 +229,21 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             user_away_timeout (float, optional): If set, set the user state as
                 "away" after this amount of time after user and agent are silent.
                 Default ``15.0`` s, set to ``None`` to disable.
-            agent_false_interruption_timeout (float, optional): If set, emit an
+            false_interruption_timeout (float, optional): If set, emit an
                 `agent_false_interruption` event after this amount of time if
                 the user is silent and no user transcript is detected after
-                the interruption. Set to ``None`` to disable. Default ``4.0`` s.
+                the interruption. Set to ``None`` to disable. Default ``2.0`` s.
+            resume_false_interruption (bool): Whether to resume the false interruption
+                after the false_interruption_timeout. Default ``True``.
             min_consecutive_speech_delay (float, optional): The minimum delay between
                 consecutive speech. Default ``0.0`` s.
             use_tts_aligned_transcript (bool, optional): Whether to use TTS-aligned
                 transcript as the input of the ``transcription_node``. Only applies
                 if ``TTS.capabilities.aligned_transcript`` is ``True`` or ``streaming``
-                is ``False``.
-            preemptive_generation (bool): Whether to use preemptive generation.
-                Default ``False``.
+                is ``False``. When NOT_GIVEN, it's disabled.
+            tts_text_transforms (Sequence[TextTransforms], optional): The transforms to apply
+                to the tts input text, available built-in transforms: ``"filter_markdown"``, ``"filter_emoji"``.
+                Set to ``None`` to disable. When NOT_GIVEN, all filters will be applied.
             preemptive_generation (bool):
                 Whether to speculatively begin LLM and TTS requests before an end-of-turn is
                 detected. When True, the agent sends inference calls as soon as a user
@@ -245,6 +259,12 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         super().__init__()
         self._loop = loop or asyncio.get_event_loop()
 
+        if is_given(agent_false_interruption_timeout):
+            logger.warning(
+                "`agent_false_interruption_timeout` is deprecated, use `false_interruption_timeout` instead"  # noqa: E501
+            )
+            false_interruption_timeout = agent_false_interruption_timeout
+
         if not is_given(video_sampler):
             video_sampler = VoiceActivityVideoSampler(speaking_fps=1.0, silent_fps=0.3)
 
@@ -261,14 +281,30 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             max_endpointing_delay=max_endpointing_delay,
             max_tool_steps=max_tool_steps,
             user_away_timeout=user_away_timeout,
-            agent_false_interruption_timeout=agent_false_interruption_timeout,
+            false_interruption_timeout=false_interruption_timeout,
+            resume_false_interruption=resume_false_interruption,
             min_consecutive_speech_delay=min_consecutive_speech_delay,
+            tts_text_transforms=(
+                tts_text_transforms
+                if is_given(tts_text_transforms)
+                else DEFAULT_TTS_TEXT_TRANSFORMS
+            ),
             preemptive_generation=preemptive_generation,
             use_tts_aligned_transcript=use_tts_aligned_transcript,
         )
         self._conn_options = conn_options or SessionConnectOptions()
         self._started = False
         self._turn_detection = turn_detection or None
+
+        if isinstance(stt, str):
+            stt = inference.STT.from_model_string(stt)
+
+        if isinstance(llm, str):
+            llm = inference.LLM.from_model_string(llm)
+
+        if isinstance(tts, str):
+            tts = inference.TTS.from_model_string(tts)
+
         self._stt = stt or None
         self._vad = vad or None
         self._llm = llm or None
@@ -304,12 +340,9 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         self._agent_state: AgentState = "initializing"
         self._user_away_timer: asyncio.TimerHandle | None = None
 
-        # used to emit the agent false interruption event
-        self._false_interruption_timer: asyncio.TimerHandle | None = None
-        self._false_interrupted_event: AgentFalseInterruptionEvent | None = None
-
         self._userdata: Userdata_T | None = userdata if is_given(userdata) else None
         self._closing_task: asyncio.Task[None] | None = None
+        self._closing: bool = False
         self._job_context_cb_registered: bool = False
 
         self._global_run_state: RunResult | None = None
@@ -387,35 +420,61 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         self.generate_reply(user_input=user_input)
         return run_state
 
+    @overload
+    async def start(
+        self,
+        agent: Agent,
+        *,
+        capture_run: Literal[True],
+        room: NotGivenOr[rtc.Room] = NOT_GIVEN,
+        room_input_options: NotGivenOr[room_io.RoomInputOptions] = NOT_GIVEN,
+        room_output_options: NotGivenOr[room_io.RoomOutputOptions] = NOT_GIVEN,
+    ) -> RunResult: ...
+
+    @overload
+    async def start(
+        self,
+        agent: Agent,
+        *,
+        capture_run: Literal[False] = False,
+        room: NotGivenOr[rtc.Room] = NOT_GIVEN,
+        room_input_options: NotGivenOr[room_io.RoomInputOptions] = NOT_GIVEN,
+        room_output_options: NotGivenOr[room_io.RoomOutputOptions] = NOT_GIVEN,
+    ) -> None: ...
+
     @tracer.start_as_current_span("agent_session", end_on_exit=False)
     async def start(
         self,
         agent: Agent,
         *,
+        capture_run: bool = False,
         room: NotGivenOr[rtc.Room] = NOT_GIVEN,
         room_input_options: NotGivenOr[room_io.RoomInputOptions] = NOT_GIVEN,
         room_output_options: NotGivenOr[room_io.RoomOutputOptions] = NOT_GIVEN,
-    ) -> None:
+    ) -> RunResult | None:
         """Start the voice agent.
 
         Create a default RoomIO if the input or output audio is not already set.
         If the console flag is provided, start a ChatCLI.
 
         Args:
+            capture_run: Whether to return a RunResult and capture the run result during session start.
             room: The room to use for input and output
             room_input_options: Options for the room input
             room_output_options: Options for the room output
         """
         async with self._lock:
             if self._started:
-                return
+                return None
 
+            self._closing = False
             self._root_span_context = otel_context.get_current()
             self._session_span = current_span = trace.get_current_span()
             current_span = trace.get_current_span()
             current_span.set_attribute(trace_types.ATTR_AGENT_LABEL, agent.label)
             current_span.set_attribute(
-                trace_types.ATTR_SESSION_OPTIONS, json.dumps(asdict(self._opts))
+                trace_types.ATTR_SESSION_OPTIONS,
+                json.dumps({k: v for k, v in asdict(self._opts).items() if is_given(v)}),
             )
 
             self._agent = agent
@@ -493,6 +552,14 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             except RuntimeError:
                 pass  # ignore
 
+            run_state: RunResult | None = None
+            if capture_run:
+                if self._global_run_state is not None and not self._global_run_state.done():
+                    raise RuntimeError("nested runs are not supported")
+
+                run_state = RunResult(output_type=None)
+                self._global_run_state = run_state
+
             # it is ok to await it directly, there is no previous task to drain
             tasks.append(
                 asyncio.create_task(self._update_activity(self._agent, wait_on_enter=False))
@@ -548,6 +615,15 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
                 " -> ".join([f"`{out.label}`" for out in audio_input]) or "(none)",
                 " -> ".join([f"`{out.label}`" for out in audio_output]) or "(none)",
             )
+            if (
+                self._opts.resume_false_interruption
+                and self.output.audio
+                and not self.output.audio.can_pause
+            ):
+                logger.warning(
+                    "resume_false_interruption is enabled but audio output does not support pause, it will be ignored",
+                    extra={"audio_output": self.output.audio.label},
+                )
 
             logger.debug(
                 "using transcript io: `AgentSession` -> %s",
@@ -561,6 +637,11 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
                     " -> ".join([f"`{out.label}`" for out in video_output]) or "(none)",
                 )
 
+            if run_state:
+                await run_state
+
+            return run_state
+
     async def drain(self) -> None:
         if self._activity is None:
             raise RuntimeError("AgentSession isn't running")
@@ -576,11 +657,13 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
     ) -> None:
         if self._closing_task:
             return
-
         self._closing_task = asyncio.create_task(
             self._aclose_impl(error=error, drain=drain, reason=reason)
         )
 
+    def shutdown(self, *, drain: bool = True) -> None:
+        self._close_soon(error=None, drain=drain, reason=CloseReason.USER_INITIATED)
+
     @utils.log_exceptions(logger=logger)
     async def _aclose_impl(
         self,
@@ -597,6 +680,9 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             if not self._started:
                 return
 
+            self._closing = True
+            self._cancel_user_away_timer()
+
             if self._activity is not None:
                 if not drain:
                     try:
@@ -606,7 +692,6 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
                         # TODO(long): force interrupt or wait for it to finish?
                         # it might be an audio played from the error callback
                         pass
-
                 await self._activity.drain()
 
                 # wait any uninterruptible speech to finish
@@ -619,6 +704,13 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
                 self.output.audio = None
                 self.output.transcription = None
 
+                if (
+                    reason != CloseReason.ERROR
+                    and (audio_recognition := self._activity._audio_recognition) is not None
+                ):
+                    # wait for the user transcript to be committed
+                    audio_recognition.commit_user_turn(audio_detached=True, transcript_timeout=2.0)
+
                 await self._activity.aclose()
                 self._activity = None
 
@@ -633,31 +725,52 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             if self._forward_audio_atask is not None:
                 await utils.aio.cancel_and_wait(self._forward_audio_atask)
 
-            if self._room_io:
-                await self._room_io.aclose()
-                self._room_io = None
-
             self._started = False
             if self._session_span:
                 self._session_span.end()
                 self._session_span = None
+
             self.emit("close", CloseEvent(error=error, reason=reason))
 
-            self._cancel_user_away_timer()
-            self._cancel_agent_false_interruption()
             self._user_state = "listening"
             self._agent_state = "initializing"
             self._llm_error_counts = 0
             self._tts_error_counts = 0
             self._root_span_context = None
 
+            # close room io after close event is emitted
+            if self._room_io:
+                await self._room_io.aclose()
+                self._room_io = None
+
         logger.debug("session closed", extra={"reason": reason.value, "error": error})
 
     async def aclose(self) -> None:
         await self._aclose_impl(reason=CloseReason.USER_INITIATED)
 
-    def update_options(self) -> None:
-        pass
+    def update_options(
+        self,
+        *,
+        min_endpointing_delay: NotGivenOr[float] = NOT_GIVEN,
+        max_endpointing_delay: NotGivenOr[float] = NOT_GIVEN,
+    ) -> None:
+        """
+        Update the options for the agent session.
+
+        Args:
+            min_endpointing_delay (NotGivenOr[float], optional): The minimum endpointing delay.
+            max_endpointing_delay (NotGivenOr[float], optional): The maximum endpointing delay.
+        """
+        if is_given(min_endpointing_delay):
+            self._opts.min_endpointing_delay = min_endpointing_delay
+        if is_given(max_endpointing_delay):
+            self._opts.max_endpointing_delay = max_endpointing_delay
+
+        if self._activity is not None:
+            self._activity.update_options(
+                min_endpointing_delay=min_endpointing_delay,
+                max_endpointing_delay=max_endpointing_delay,
+            )
 
     def say(
         self,
@@ -671,11 +784,18 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             raise RuntimeError("AgentSession isn't running")
 
         run_state = self._global_run_state
-        if self._activity.scheduling_paused:
-            if self._next_activity is None:
-                raise RuntimeError("AgentSession is closing, cannot use say()")
+        activity = self._next_activity if self._activity.scheduling_paused else self._activity
 
-            handle = self._next_activity.say(
+        if activity is None:
+            raise RuntimeError("AgentSession is closing, cannot use say()")
+
+        # attach to the session span if called outside of the AgentSession
+        use_span: AbstractContextManager[trace.Span | None] = nullcontext()
+        if trace.get_current_span() is trace.INVALID_SPAN and self._session_span is not None:
+            use_span = trace.use_span(self._session_span, end_on_exit=False)
+
+        with use_span:
+            handle = activity.say(
                 text,
                 audio=audio,
                 allow_interruptions=allow_interruptions,
@@ -684,17 +804,6 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             if run_state:
                 run_state._watch_handle(handle)
 
-            return handle
-
-        handle = self._activity.say(
-            text,
-            audio=audio,
-            allow_interruptions=allow_interruptions,
-            add_to_chat_ctx=add_to_chat_ctx,
-        )
-        if run_state:
-            run_state._watch_handle(handle)
-
         return handle
 
     def generate_reply(
@@ -728,11 +837,18 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         )
 
         run_state = self._global_run_state
-        if self._activity.scheduling_paused:
-            if self._next_activity is None:
-                raise RuntimeError("AgentSession is closing, cannot use generate_reply()")
+        activity = self._next_activity if self._activity.scheduling_paused else self._activity
 
-            handle = self._next_activity._generate_reply(
+        if activity is None:
+            raise RuntimeError("AgentSession is closing, cannot use generate_reply()")
+
+        # attach to the session span if called outside of the AgentSession
+        use_span: AbstractContextManager[trace.Span | None] = nullcontext()
+        if trace.get_current_span() is trace.INVALID_SPAN and self._session_span is not None:
+            use_span = trace.use_span(self._session_span, end_on_exit=False)
+
+        with use_span:
+            handle = activity._generate_reply(
                 user_message=user_message,
                 instructions=instructions,
                 tool_choice=tool_choice,
@@ -741,20 +857,9 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             if run_state:
                 run_state._watch_handle(handle)
 
-            return handle
-
-        handle = self._activity._generate_reply(
-            user_message=user_message,
-            instructions=instructions,
-            tool_choice=tool_choice,
-            allow_interruptions=allow_interruptions,
-        )
-        if run_state:
-            run_state._watch_handle(handle)
-
         return handle
 
-    def interrupt(self) -> asyncio.Future[None]:
+    def interrupt(self, *, force: bool = False) -> asyncio.Future[None]:
         """Interrupt the current speech generation.
 
         Returns:
@@ -764,7 +869,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         if self._activity is None:
             raise RuntimeError("AgentSession isn't running")
 
-        return self._activity.interrupt()
+        return self._activity.interrupt(force=force)
 
     def clear_user_turn(self) -> None:
         # clear the transcription or input audio buffer of the user turn
@@ -773,13 +878,17 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
 
         self._activity.clear_user_turn()
 
-    def commit_user_turn(self, *, transcript_timeout: float = 2.0) -> None:
+    def commit_user_turn(
+        self, *, transcript_timeout: float = 2.0, stt_flush_duration: float = 2.0
+    ) -> None:
         """Commit the user turn and generate a reply.
 
         Args:
             transcript_timeout (float, optional): The timeout for the final transcript
                 to be received after committing the user turn.
                 Increase this value if the STT is slow to respond.
+            stt_flush_duration (float, optional): The duration of the silence to be appended to the STT
+                to flush the buffer and generate the final transcript.
 
         Raises:
             RuntimeError: If the AgentSession isn't running.
@@ -787,14 +896,17 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         if self._activity is None:
             raise RuntimeError("AgentSession isn't running")
 
-        self._activity.commit_user_turn(transcript_timeout=transcript_timeout)
+        self._activity.commit_user_turn(
+            transcript_timeout=transcript_timeout, stt_flush_duration=stt_flush_duration
+        )
 
     def update_agent(self, agent: Agent) -> None:
         self._agent = agent
 
         if self._started:
             self._update_activity_atask = task = asyncio.create_task(
-                self._update_activity_task(self._agent), name="_update_activity_task"
+                self._update_activity_task(self._update_activity_atask, self._agent),
+                name="_update_activity_task",
             )
             run_state = self._global_run_state
             if run_state:
@@ -812,7 +924,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         wait_on_enter: bool = True,
     ) -> None:
         async with self._activity_lock:
-            # _update_activity is called directy sometimes, update for redundancy
+            # _update_activity is called directly sometimes, update for redundancy
             self._agent = agent
 
             if new_activity == "start":
@@ -826,6 +938,11 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
 
                 self._next_activity = agent._activity
 
+            if self._root_span_context is not None:
+                # restore the root span context so on_exit, on_enter, and future turns
+                # are direct children of the root span, not nested under a tool call.
+                otel_context.attach(self._root_span_context)
+
             previous_activity_v = self._activity
             if self._activity is not None:
                 if previous_activity == "close":
@@ -849,18 +966,19 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
             elif new_activity == "resume":
                 await self._activity.resume()
 
-            if wait_on_enter:
-                assert self._activity._on_enter_task is not None
-                await asyncio.shield(self._activity._on_enter_task)
+        # move it outside the lock to allow calling _update_activity in on_enter of a new agent
+        if wait_on_enter:
+            assert self._activity._on_enter_task is not None
+            await asyncio.shield(self._activity._on_enter_task)
 
     @utils.log_exceptions(logger=logger)
-    async def _update_activity_task(self, task: Agent) -> None:
-        if self._root_span_context is not None:
-            # restore the root span context so on_exit, on_enter, and future turns
-            # are direct children of the root span, not nested under a tool call.
-            otel_context.attach(self._root_span_context)
+    async def _update_activity_task(
+        self, old_task: asyncio.Task[None] | None, agent: Agent
+    ) -> None:
+        if old_task is not None:
+            await old_task
 
-        await self._update_activity(task)
+        await self._update_activity(agent)
 
     def _on_error(
         self,
@@ -954,9 +1072,6 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         else:
             self._cancel_user_away_timer()
 
-        if state != "listening":
-            self._cancel_agent_false_interruption()
-
         old_state = self._agent_state
         self._agent_state = state
         self.emit(
@@ -976,7 +1091,7 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         elif self._user_speaking_span is not None:
             end_time = last_speaking_time or time.time()
             self._user_speaking_span.set_attribute(trace_types.ATTR_END_TIME, end_time)
-            self._user_speaking_span.end(end_time=int(end_time * 1_000_000_000))  # nanoseconds
+            self._user_speaking_span.end()
             self._user_speaking_span = None
 
         if state == "listening" and self._agent_state == "listening":
@@ -984,14 +1099,6 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         else:
             self._cancel_user_away_timer()
 
-        # pause the false interruption timer if user is speaking and recreate it after user stops
-        if state == "speaking" and self._false_interruption_timer:
-            ev = self._false_interrupted_event
-            self._cancel_agent_false_interruption()
-            self._false_interrupted_event = ev
-        elif state == "listening" and self._false_interrupted_event:
-            self._schedule_agent_false_interruption(self._false_interrupted_event)
-
         old_state = self._user_state
         self._user_state = state
         self.emit(
@@ -1000,37 +1107,18 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         )
 
     def _user_input_transcribed(self, ev: UserInputTranscribedEvent) -> None:
+        if self.user_state == "away" and ev.is_final:
+            # reset user state from away to listening in case VAD has a miss detection
+            self._update_user_state("listening")
+
         self.emit("user_input_transcribed", ev)
-        if ev.is_final:
-            # fully cancel the false interruption event if user transcript arrives
-            self._cancel_agent_false_interruption()
 
     def _conversation_item_added(self, message: llm.ChatMessage) -> None:
         self._chat_ctx.insert(message)
         self.emit("conversation_item_added", ConversationItemAddedEvent(item=message))
 
-    def _schedule_agent_false_interruption(self, ev: AgentFalseInterruptionEvent) -> None:
-        if self._opts.agent_false_interruption_timeout is None:
-            return
-
-        def _emit_event() -> None:
-            if self._agent_state != "listening" or self._user_state != "listening":
-                return
-
-            self.emit("agent_false_interruption", ev)
-            self._false_interruption_timer = None
-
-        self._cancel_agent_false_interruption()
-        self._false_interruption_timer = self._loop.call_later(
-            self._opts.agent_false_interruption_timeout, _emit_event
-        )
-        self._false_interrupted_event = ev
-
-    def _cancel_agent_false_interruption(self) -> None:
-        if self._false_interruption_timer is not None:
-            self._false_interruption_timer.cancel()
-            self._false_interruption_timer = None
-        self._false_interrupted_event = None
+    def _tool_items_added(self, items: Sequence[llm.FunctionCall | llm.FunctionCallOutput]) -> None:
+        self._chat_ctx.insert(items)
 
     # move them to the end to avoid shadowing the same named modules for mypy
     @property
@@ -1077,7 +1165,16 @@ class AgentSession(rtc.EventEmitter[EventTypes], Generic[Userdata_T]):
         pass
 
     def _on_audio_output_changed(self) -> None:
-        pass
+        if (
+            self._started
+            and self._opts.resume_false_interruption
+            and (audio_output := self.output.audio)
+            and not audio_output.can_pause
+        ):
+            logger.warning(
+                "resume_false_interruption is enabled, but the audio output does not support pause, ignored",
+                extra={"audio_output": audio_output.label},
+            )
 
     def _on_text_output_changed(self) -> None:
         pass
