@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import json
+import math
 import time
 from collections.abc import AsyncIterable
 from dataclasses import dataclass
@@ -14,7 +15,8 @@ from livekit import rtc
 from .. import llm, stt, utils, vad
 from ..log import logger
 from ..telemetry import trace_types, tracer
-from ..utils import aio
+from ..types import NOT_GIVEN, NotGivenOr
+from ..utils import aio, is_given
 from . import io
 from .agent import ModelSettings
 
@@ -41,18 +43,28 @@ class _PreemptiveGenerationInfo:
 
 
 class _TurnDetector(Protocol):
+    @property
+    def model(self) -> str:
+        return "unknown"
+
+    @property
+    def provider(self) -> str:
+        return "unknown"
+
     # TODO: Move those two functions to EOU ctor (capabilities dataclass)
     async def unlikely_threshold(self, language: str | None) -> float | None: ...
     async def supports_language(self, language: str | None) -> bool: ...
 
-    async def predict_end_of_turn(self, chat_ctx: llm.ChatContext) -> float: ...
+    async def predict_end_of_turn(
+        self, chat_ctx: llm.ChatContext, *, timeout: float | None = None
+    ) -> float: ...
 
 
 class RecognitionHooks(Protocol):
-    def on_start_of_speech(self, ev: vad.VADEvent) -> None: ...
+    def on_start_of_speech(self, ev: vad.VADEvent | None) -> None: ...
     def on_vad_inference_done(self, ev: vad.VADEvent) -> None: ...
-    def on_end_of_speech(self, ev: vad.VADEvent) -> None: ...
-    def on_interim_transcript(self, ev: stt.SpeechEvent) -> None: ...
+    def on_end_of_speech(self, ev: vad.VADEvent | None) -> None: ...
+    def on_interim_transcript(self, ev: stt.SpeechEvent, *, speaking: bool | None) -> None: ...
     def on_final_transcript(self, ev: stt.SpeechEvent) -> None: ...
     def on_end_of_turn(self, info: _EndOfTurnInfo) -> bool: ...
     def on_preemptive_generation(self, info: _PreemptiveGenerationInfo) -> None: ...
@@ -91,10 +103,13 @@ class AudioRecognition:
         self._speaking = False
         self._last_speaking_time: float = 0
         self._last_final_transcript_time: float = 0
+        # used for manual commit_user_turn
         self._final_transcript_received = asyncio.Event()
         self._final_transcript_confidence: list[float] = []
         self._audio_transcript = ""
         self._audio_interim_transcript = ""
+        # used for STTs that support preflight mode, so it could start preemptive generation earlier
+        self._audio_preflight_transcript = ""
         self._last_language: str | None = None
 
         self._stt_ch: aio.Chan[rtc.AudioFrame] | None = None
@@ -102,6 +117,18 @@ class AudioRecognition:
         self._tasks: set[asyncio.Task[Any]] = set()
 
         self._user_turn_span: trace.Span | None = None
+        self._closing = asyncio.Event()
+
+    def update_options(
+        self,
+        *,
+        min_endpointing_delay: NotGivenOr[float] = NOT_GIVEN,
+        max_endpointing_delay: NotGivenOr[float] = NOT_GIVEN,
+    ) -> None:
+        if is_given(min_endpointing_delay):
+            self._min_endpointing_delay = min_endpointing_delay
+        if is_given(max_endpointing_delay):
+            self._max_endpointing_delay = max_endpointing_delay
 
     def start(self) -> None:
         self.update_stt(self._stt)
@@ -120,9 +147,12 @@ class AudioRecognition:
             self._vad_ch.send_nowait(frame)
 
     async def aclose(self) -> None:
-        await aio.cancel_and_wait(*self._tasks)
+        self._closing.set()
+
         if self._commit_user_turn_atask is not None:
-            await aio.cancel_and_wait(self._commit_user_turn_atask)
+            await self._commit_user_turn_atask
+
+        await aio.cancel_and_wait(*self._tasks)
 
         if self._stt_atask is not None:
             await aio.cancel_and_wait(self._stt_atask)
@@ -131,7 +161,7 @@ class AudioRecognition:
             await aio.cancel_and_wait(self._vad_atask)
 
         if self._end_of_turn_task is not None:
-            await aio.cancel_and_wait(self._end_of_turn_task)
+            await self._end_of_turn_task
 
     def update_stt(self, stt: io.STTNode | None) -> None:
         self._stt = stt
@@ -164,6 +194,7 @@ class AudioRecognition:
     def clear_user_turn(self) -> None:
         self._audio_transcript = ""
         self._audio_interim_transcript = ""
+        self._audio_preflight_transcript = ""
         self._final_transcript_confidence = []
         self._user_turn_committed = False
 
@@ -172,7 +203,16 @@ class AudioRecognition:
         self.update_stt(None)
         self.update_stt(stt)
 
-    def commit_user_turn(self, *, audio_detached: bool, transcript_timeout: float) -> None:
+    def commit_user_turn(
+        self,
+        *,
+        audio_detached: bool,
+        transcript_timeout: float,
+        stt_flush_duration: float = 2.0,
+    ) -> None:
+        if not self._stt or self._closing.is_set():
+            return
+
         async def _commit_user_turn() -> None:
             if time.time() - self._last_final_transcript_time > 0.5:
                 # if the last final transcript is received more than 0.5s ago
@@ -189,7 +229,8 @@ class AudioRecognition:
                         num_channels=1,
                         samples_per_channel=num_samples,
                     )
-                    for _ in range(5):  # 5 * 0.2s = 1s
+                    num_frames = max(0, int(math.ceil(stt_flush_duration / silence_frame.duration)))
+                    for _ in range(num_frames):
                         self.push_audio(silence_frame)
 
                 # wait for the final transcript to be available
@@ -199,13 +240,31 @@ class AudioRecognition:
                         timeout=transcript_timeout,
                     )
                 except asyncio.TimeoutError:
-                    pass
+                    if self._audio_interim_transcript:
+                        logger.warning(
+                            "final transcript not received after timeout",
+                            extra={
+                                "transcript_timeout": transcript_timeout,
+                                "interim_transcript": self._audio_interim_transcript,
+                            },
+                        )
 
             if self._audio_interim_transcript:
+                # emit interim transcript as final for frontend display
+                self._hooks.on_final_transcript(
+                    stt.SpeechEvent(
+                        type=stt.SpeechEventType.FINAL_TRANSCRIPT,
+                        alternatives=[
+                            stt.SpeechData(language="", text=self._audio_interim_transcript)
+                        ],
+                    )
+                )
+
                 # append interim transcript in case the final transcript is not ready
                 self._audio_transcript = (
                     f"{self._audio_transcript} {self._audio_interim_transcript}".strip()
                 )
+
             self._audio_interim_transcript = ""
             chat_ctx = self._hooks.retrieve_chat_ctx().copy()
             self._run_eou_detection(chat_ctx)
@@ -240,7 +299,61 @@ class AudioRecognition:
             return
 
         if ev.type == stt.SpeechEventType.FINAL_TRANSCRIPT:
+            transcript = ev.alternatives[0].text
+            language = ev.alternatives[0].language
+            confidence = ev.alternatives[0].confidence
+
+            if not self._last_language or (
+                language and len(transcript) > MIN_LANGUAGE_DETECTION_LENGTH
+            ):
+                self._last_language = language
+
+            if not transcript:
+                return
+
             self._hooks.on_final_transcript(ev)
+            logger.debug(
+                "received user transcript",
+                extra={"user_transcript": transcript, "language": self._last_language},
+            )
+
+            self._last_final_transcript_time = time.time()
+            self._audio_transcript += f" {transcript}"
+            self._audio_transcript = self._audio_transcript.lstrip()
+            self._final_transcript_confidence.append(confidence)
+            transcript_changed = self._audio_transcript != self._audio_preflight_transcript
+            self._audio_interim_transcript = ""
+            self._audio_preflight_transcript = ""
+            self._final_transcript_received.set()
+
+            if not self._vad or self._last_speaking_time == 0:
+                # vad disabled, use stt timestamp
+                # TODO: this would screw up transcription latency metrics
+                # but we'll live with it for now.
+                # the correct way is to ensure STT fires SpeechEventType.END_OF_SPEECH
+                # and using that timestamp for _last_speaking_time
+                self._last_speaking_time = time.time()
+
+            if self._vad_base_turn_detection or self._user_turn_committed:
+                if transcript_changed:
+                    self._hooks.on_preemptive_generation(
+                        _PreemptiveGenerationInfo(
+                            new_transcript=self._audio_transcript,
+                            transcript_confidence=(
+                                sum(self._final_transcript_confidence)
+                                / len(self._final_transcript_confidence)
+                                if self._final_transcript_confidence
+                                else 0
+                            ),
+                        )
+                    )
+
+                if not self._speaking:
+                    chat_ctx = self._hooks.retrieve_chat_ctx().copy()
+                    self._run_eou_detection(chat_ctx)
+
+        elif ev.type == stt.SpeechEventType.PREFLIGHT_TRANSCRIPT:
+            self._hooks.on_interim_transcript(ev, speaking=self._speaking if self._vad else None)
             transcript = ev.alternatives[0].text
             language = ev.alternatives[0].language
             confidence = ev.alternatives[0].confidence
@@ -254,52 +367,53 @@ class AudioRecognition:
                 return
 
             logger.debug(
-                "received user transcript",
+                "received user preflight transcript",
                 extra={"user_transcript": transcript, "language": self._last_language},
             )
 
+            # still need to increment it as it's used for turn detection,
             self._last_final_transcript_time = time.time()
-            self._audio_transcript += f" {transcript}"
-            self._audio_transcript = self._audio_transcript.lstrip()
-            self._final_transcript_confidence.append(confidence)
-            self._audio_interim_transcript = ""
-            self._final_transcript_received.set()
+            # preflight transcript includes all pre-committed transcripts (including final transcript from the previous STT run)
+            self._audio_preflight_transcript = (self._audio_transcript + " " + transcript).lstrip()
+            self._audio_interim_transcript = transcript
 
             if not self._vad or self._last_speaking_time == 0:
                 # vad disabled, use stt timestamp
-                # TODO: this would screw up transcription latency metrics
-                # but we'll live with it for now.
-                # the correct way is to ensure STT fires SpeechEventType.END_OF_SPEECH
-                # and using that timestamp for _last_speaking_time
                 self._last_speaking_time = time.time()
 
-            if self._vad_base_turn_detection or self._user_turn_committed:
+            if self._turn_detection_mode != "manual" or self._user_turn_committed:
+                confidence_vals = list(self._final_transcript_confidence) + [confidence]
                 self._hooks.on_preemptive_generation(
                     _PreemptiveGenerationInfo(
-                        new_transcript=self._audio_transcript,
-                        transcript_confidence=(
-                            sum(self._final_transcript_confidence)
-                            / len(self._final_transcript_confidence)
-                            if self._final_transcript_confidence
-                            else 0
-                        ),
+                        new_transcript=self._audio_preflight_transcript,
+                        transcript_confidence=sum(confidence_vals) / len(confidence_vals),
                     )
                 )
 
-                if not self._speaking:
-                    chat_ctx = self._hooks.retrieve_chat_ctx().copy()
-                    self._run_eou_detection(chat_ctx)
-
         elif ev.type == stt.SpeechEventType.INTERIM_TRANSCRIPT:
-            self._hooks.on_interim_transcript(ev)
+            self._hooks.on_interim_transcript(ev, speaking=self._speaking if self._vad else None)
             self._audio_interim_transcript = ev.alternatives[0].text
 
         elif ev.type == stt.SpeechEventType.END_OF_SPEECH and self._turn_detection_mode == "stt":
+            with trace.use_span(self._ensure_user_turn_span()):
+                self._hooks.on_end_of_speech(None)
+
+            self._speaking = False
             self._user_turn_committed = True
-            if not self._speaking:
-                # start response after vad fires END_OF_SPEECH to avoid vad interruption
-                chat_ctx = self._hooks.retrieve_chat_ctx().copy()
-                self._run_eou_detection(chat_ctx)
+            self._last_speaking_time = time.time()
+
+            chat_ctx = self._hooks.retrieve_chat_ctx().copy()
+            self._run_eou_detection(chat_ctx)
+
+        elif ev.type == stt.SpeechEventType.START_OF_SPEECH and self._turn_detection_mode == "stt":
+            with trace.use_span(self._ensure_user_turn_span()):
+                self._hooks.on_start_of_speech(None)
+
+            self._speaking = True
+            self._last_speaking_time = time.time()
+
+            if self._end_of_turn_task is not None:
+                self._end_of_turn_task.cancel()
 
     async def _on_vad_event(self, ev: vad.VADEvent) -> None:
         if ev.type == vad.VADEventType.START_OF_SPEECH:
@@ -390,7 +504,11 @@ class AudioRecognition:
                         )
 
             extra_sleep = last_speaking_time + endpointing_delay - time.time()
-            await asyncio.sleep(max(extra_sleep, 0))
+            if extra_sleep > 0:
+                try:
+                    await asyncio.wait_for(self._closing.wait(), timeout=extra_sleep)
+                except asyncio.TimeoutError:
+                    pass
 
             confidence_avg = (
                 sum(self._final_transcript_confidence) / len(self._final_transcript_confidence)
