@@ -4,7 +4,7 @@ import asyncio
 import functools
 import inspect
 import json
-from collections.abc import AsyncIterable
+from collections.abc import AsyncIterable, Sequence
 from dataclasses import dataclass, field
 from typing import TYPE_CHECKING, Any, Callable, Optional, Protocol, runtime_checkable
 
@@ -36,6 +36,7 @@ from .speech_handle import SpeechHandle
 if TYPE_CHECKING:
     from .agent import Agent, ModelSettings
     from .agent_session import AgentSession
+    from .transcription.filters import TextTransforms
 
 
 @runtime_checkable
@@ -173,12 +174,21 @@ class _TTSGenerationData:
 
 
 def perform_tts_inference(
-    *, node: io.TTSNode, input: AsyncIterable[str], model_settings: ModelSettings
+    *,
+    node: io.TTSNode,
+    input: AsyncIterable[str],
+    model_settings: ModelSettings,
+    text_transforms: Sequence[TextTransforms] | None,
 ) -> tuple[asyncio.Task[bool], _TTSGenerationData]:
     audio_ch = aio.Chan[rtc.AudioFrame]()
     timed_texts_fut = asyncio.Future[Optional[aio.Chan[io.TimedString]]]()
     data = _TTSGenerationData(audio_ch=audio_ch, timed_texts_fut=timed_texts_fut)
 
+    if text_transforms:
+        from .transcription.filters import apply_text_transforms
+
+        input = apply_text_transforms(input, text_transforms)
+
     tts_task = asyncio.create_task(_tts_inference_task(node, input, model_settings, data))
 
     def _inference_done(_: asyncio.Task[bool]) -> None:
@@ -280,6 +290,7 @@ async def _audio_forwarding_task(
 ) -> None:
     resampler: rtc.AudioResampler | None = None
     try:
+        audio_output.resume()
         async for frame in tts_output:
             out.audio.append(frame)
 
@@ -305,6 +316,11 @@ async def _audio_forwarding_task(
             # (after completing the first frame)
             if not out.first_frame_fut.done():
                 out.first_frame_fut.set_result(None)
+
+        if resampler:
+            for frame in resampler.flush():
+                await audio_output.capture_frame(frame)
+
     finally:
         if isinstance(tts_output, _ACloseable):
             try:
@@ -312,13 +328,6 @@ async def _audio_forwarding_task(
             except Exception as e:
                 logger.error("error while closing tts output", exc_info=e)
 
-        if resampler:
-            try:
-                for frame in resampler.flush():
-                    await audio_output.capture_frame(frame)
-            except Exception as e:
-                logger.error("error while flushing resampler", exc_info=e)
-
         audio_output.flush()
 
 
@@ -517,10 +526,11 @@ async def _execute_tools_task(
                         val = await function_callable()
                         output = make_tool_output(fnc_call=fnc_call, output=val, exception=None)
                     except BaseException as e:
-                        logger.exception(
-                            "exception occurred while executing tool",
-                            extra={"function": fnc_call.name, "speech_id": speech_handle.id},
-                        )
+                        if not isinstance(e, StopResponse):
+                            logger.exception(
+                                "exception occurred while executing tool",
+                                extra={"function": fnc_call.name, "speech_id": speech_handle.id},
+                            )
 
                         output = make_tool_output(fnc_call=fnc_call, output=None, exception=e)
 
