@@ -58,6 +58,7 @@ from .version import __version__
 ASSIGNMENT_TIMEOUT = 7.5
 UPDATE_STATUS_INTERVAL = 2.5
 UPDATE_LOAD_INTERVAL = 0.5
+HEARTBEAT_INTERVAL = 30
 
 
 def _default_initialize_process_fnc(proc: JobProcess) -> Any:
@@ -227,6 +228,11 @@ class WorkerOptions:
     """
     prometheus_port: NotGivenOr[int] = NOT_GIVEN
     """When enabled, will expose prometheus metrics on :{prometheus_port}/metrics"""
+    prometheus_multiproc_dir: str | None = None
+    """Directory for prometheus multiprocess mode to enable metrics collection from child job processes.
+    When set, the PROMETHEUS_MULTIPROC_DIR environment variable will be configured automatically.
+    When None (default), multiprocess mode is disabled and only main process metrics are collected.
+    Users can also set PROMETHEUS_MULTIPROC_DIR environment variable directly before starting the worker."""
 
     def validate_config(self, devmode: bool) -> None:
         load_threshold = _WorkerEnvOption.getvalue(self.load_threshold, devmode)
@@ -254,10 +260,10 @@ class Worker(utils.EventEmitter[EventTypes]):
         loop: asyncio.AbstractEventLoop | None = None,
     ) -> None:
         super().__init__()
-        opts.ws_url = opts.ws_url or os.environ.get("LIVEKIT_URL") or ""
-        opts.api_key = opts.api_key or os.environ.get("LIVEKIT_API_KEY") or ""
-        opts.api_secret = opts.api_secret or os.environ.get("LIVEKIT_API_SECRET") or ""
-        opts._worker_token = os.environ.get("LIVEKIT_WORKER_TOKEN") or None
+        opts.ws_url = opts.ws_url or os.environ.get("LIVEKIT_URL", "")
+        opts.api_key = opts.api_key or os.environ.get("LIVEKIT_API_KEY", "")
+        opts.api_secret = opts.api_secret or os.environ.get("LIVEKIT_API_SECRET", "")
+        opts._worker_token = os.environ.get("LIVEKIT_WORKER_TOKEN", None)
 
         if not opts.ws_url:
             raise ValueError("ws_url is required, or add LIVEKIT_URL in your environment")
@@ -270,6 +276,13 @@ class Worker(utils.EventEmitter[EventTypes]):
                 "api_secret is required, or add LIVEKIT_API_SECRET in your environment"
             )
 
+        # Ensure job processes and thread executors can initialize JobContext.api
+        # by inheriting credentials from environment variables. This guarantees that
+        # ctx.api works even when credentials are only supplied via WorkerOptions.
+        os.environ["LIVEKIT_URL"] = opts.ws_url
+        os.environ["LIVEKIT_API_KEY"] = opts.api_key
+        os.environ["LIVEKIT_API_SECRET"] = opts.api_secret
+
         if opts.job_memory_limit_mb > 0 and opts.job_executor_type != JobExecutorType.PROCESS:
             logger.warning(
                 "max_job_memory_usage is only supported for process-based job executors, "
@@ -368,6 +381,21 @@ class Worker(utils.EventEmitter[EventTypes]):
         self._http_server.app.add_routes([web.get("/worker", worker)])
 
         self._prometheus_server: telemetry.http_server.HttpServer | None = None
+        self._prometheus_multiproc_dir: str | None = None
+
+        # Setup prometheus multiprocess mode if explicitly configured
+        if opts.prometheus_multiproc_dir:
+            self._prometheus_multiproc_dir = opts.prometheus_multiproc_dir
+            # Set environment variable for prometheus multiprocess mode
+            os.environ["PROMETHEUS_MULTIPROC_DIR"] = self._prometheus_multiproc_dir
+        elif "PROMETHEUS_MULTIPROC_DIR" in os.environ:
+            # Use existing environment variable if already set
+            self._prometheus_multiproc_dir = os.environ["PROMETHEUS_MULTIPROC_DIR"]
+
+        # Create prometheus directory if it doesn't exist
+        if self._prometheus_multiproc_dir:
+            os.makedirs(self._prometheus_multiproc_dir, exist_ok=True)
+
         if is_given(self._opts.prometheus_port):
             self._prometheus_server = telemetry.http_server.HttpServer(
                 opts.host, self._opts.prometheus_port, loop=self._loop
@@ -391,6 +419,21 @@ class Worker(utils.EventEmitter[EventTypes]):
             extra={"version": __version__, "rtc-version": rtc.__version__},
         )
 
+        # Clean prometheus multiprocess directory to avoid stale metrics (only if it exists)
+        if self._prometheus_multiproc_dir and os.path.exists(self._prometheus_multiproc_dir):
+            logger.debug(
+                "cleaning prometheus multiprocess directory",
+                extra={"dir": self._prometheus_multiproc_dir},
+            )
+            # Remove all files in the directory but keep the directory itself
+            for filename in os.listdir(self._prometheus_multiproc_dir):
+                file_path = os.path.join(self._prometheus_multiproc_dir, filename)
+                try:
+                    if os.path.isfile(file_path):
+                        os.unlink(file_path)
+                except Exception as e:
+                    logger.warning(f"failed to remove {file_path}: {e}")
+
         if self._opts.multiprocessing_context == "forkserver":
             plugin_packages = [p.package for p in Plugin.registered_plugins] + ["av"]
             logger.info("preloading plugins", extra={"packages": plugin_packages})
@@ -441,6 +484,10 @@ class Worker(utils.EventEmitter[EventTypes]):
 
                 self._worker_load = await asyncio.get_event_loop().run_in_executor(None, load_fnc)
 
+                # Update child process count metric for prometheus multiprocess mode
+                if self._prometheus_multiproc_dir:
+                    telemetry.metrics._update_child_proc_count()
+
                 load_threshold = _WorkerEnvOption.getvalue(self._opts.load_threshold, self._devmode)
                 default_num_idle_processes = _WorkerEnvOption.getvalue(
                     self._opts.num_idle_processes, self._devmode
@@ -494,7 +541,7 @@ class Worker(utils.EventEmitter[EventTypes]):
         await self._update_worker_status()
 
         async def _join_jobs() -> None:
-            for proc in self._proc_pool.processes:
+            for proc in list(self._proc_pool.processes):
                 if proc.running_job:
                     await proc.join()
 
@@ -650,6 +697,7 @@ class Worker(utils.EventEmitter[EventTypes]):
                     headers=headers,
                     params=params,
                     autoping=True,
+                    heartbeat=HEARTBEAT_INTERVAL,
                     proxy=self._opts.http_proxy or None,
                 )
 
